@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -530,7 +530,8 @@ class StateForActualGivenExecution(object):
                                 "slower than this into an error, or you can set "
                                 "it to None to disable this check entirely."
                             )
-                            % (self.test.__name__, runtime, ceil(runtime / 100) * 100)
+                            % (self.test.__name__, runtime, ceil(runtime / 100) * 100),
+                            since="2017-11-20",
                         )
                 else:
                     current_deadline = self.settings.deadline
@@ -621,17 +622,23 @@ class StateForActualGivenExecution(object):
             raise
         except EXCEPTIONS_TO_FAIL as e:
             escalate_hypothesis_internal_error()
-            tb = get_trimmed_traceback()
-            data.__expected_traceback = "".join(
-                traceback.format_exception(type(e), e, tb)
-            )
-            data.__expected_exception = e
-            verbose_report(data.__expected_traceback)
+            if data.frozen:
+                # This can happen if an error occurred in a finally
+                # block somewhere, suppressing our original StopTest.
+                # We raise a new one here to resume normal operation.
+                raise StopTest(data.testcounter)
+            else:
+                tb = get_trimmed_traceback()
+                data.__expected_traceback = "".join(
+                    traceback.format_exception(type(e), e, tb)
+                )
+                data.__expected_exception = e
+                verbose_report(data.__expected_traceback)
 
-            origin = traceback.extract_tb(tb)[-1]
-            filename = origin[0]
-            lineno = origin[1]
-            data.mark_interesting((type(e), filename, lineno))
+                origin = traceback.extract_tb(tb)[-1]
+                filename = origin[0]
+                lineno = origin[1]
+                data.mark_interesting((type(e), filename, lineno))
 
     def run(self):
         # Tell pytest to omit the body of this function from tracebacks
@@ -665,7 +672,8 @@ class StateForActualGivenExecution(object):
                         "To get the future behaviour, update your settings to "
                         "include database=None."
                     )
-                    % (self.test.__name__,)
+                    % (self.test.__name__,),
+                    since="2017-11-29",
                 )
             if self.__had_seed:
                 note_deprecation(
@@ -675,7 +683,8 @@ class StateForActualGivenExecution(object):
                         "from the database. To get the future behaviour, update "
                         "your settings for this test to include database=None."
                     )
-                    % (self.test.__name__,)
+                    % (self.test.__name__,),
+                    since="2017-11-29",
                 )
 
         timed_out = runner.exit_reason == ExitReason.timeout
@@ -893,7 +902,8 @@ def given(
                         "@given(booleans()) @given(integers()), you could write "
                         "@given(booleans(), integers())"
                     )
-                    % (test.__name__,)
+                    % (test.__name__,),
+                    since="2018-09-01",
                 )
 
             settings = wrapped_test._hypothesis_internal_use_settings
